#include "reset.h"
#include "utils/timebase.h"
#include "com/debug.h"


/** perform a software reset. */
void __attribute__((noreturn))
SW_Reset(void)
{
	dbg("------ SOFTWARE RESET ------");

	// wait a while. Don't use timebase, may be called from task.
	for(uint32_t i = 0; i < 20000; i++) {
		if (com_tx_done(debug_iface)) break;
	}

	NVIC_SystemReset();
	while(1); // just to satisfy the clang analysis
}


/**
 * prints status code and perform software reset.
 *
 * This overrides a newlib syscall.
 */
void __attribute__((noreturn))
exit(int val)
{
	error("EXIT: %d", val);
	SW_Reset();
}


/** Get reset reason byte */
uint8_t get_reset_reason_raw(void)
{
	return (uint8_t)(RCC->CSR >> 24);
}


/** Get reset reason enum */
ResetReason get_reset_reason(void)
{
	uint8_t reason = get_reset_reason_raw();

	if (reason & RST_LPWR) return RST_LPWR;
	if (reason & RST_WWDG) return RST_WWDG;
	if (reason & RST_IWDG) return RST_IWDG;
	if (reason & RST_SOFT) return RST_SOFT;
	if (reason & RST_OBL) return RST_OBL;
	if (reason == RST_START) return RST_START;
	if (reason == RST_PIN) return RST_PIN;
	if (reason & RST_POR) return RST_POR;

	return RST_UNKNOWN;
}


/** Get reset reason as string */
const char *get_reset_reason_str(void)
{
	switch (get_reset_reason()) {
		case RST_LPWR: return "LOW_POWER_RESET";
		case RST_WWDG: return "WWDG";
		case RST_IWDG: return "IWDG";
		case RST_SOFT: return "SOFT";
		case RST_OBL: return "OBL";
		case RST_PIN: return "PIN";
		case RST_POR: return "POR";
		case RST_START: return "STARTUP";
		case RST_UNKNOWN:
		default:
			return "UNKNOWN";
	}
}

/** Clear the reset reason bits */
void clear_reset_reason(void)
{
	RCC->CSR |= RCC_CSR_RMVF;
}

